#  tests for clr_loader-0.2.5-py39hca03da5_0 (this is a generated file);
print('===== testing package: clr_loader-0.2.5-py39hca03da5_0 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
try:
    from importlib.resources import files as resources_files
except ImportError:
    from importlib_resources import files as resources_files
import sys

import clr_loader


# Check if DLLs exist
base_path = resources_files('clr_loader') / 'ffi' / 'dlls'
for arch in ['amd64', 'x86']:
    for ext in ['dll', 'pdb']:
        if not (base_path / arch / f'ClrLoader.{ext}').exists():
            raise FileNotFoundError(f'DLL ffi/dlls/{arch}/ClrLoader.{ext} not found in package.')
print("DLLs found; test passed.")
#  --- run_test.py (end) ---

print('===== clr_loader-0.2.5-py39hca03da5_0 OK =====');
